﻿using Azure.Messaging.ServiceBus;
using Azure.Storage.Blobs;

using Microsoft.Azure.WebJobs;
using Microsoft.Azure.WebJobs.Extensions.SignalRService;
using Microsoft.Extensions.Logging;

using Newtonsoft.Json;

using System;
using System.Threading.Tasks;

namespace TemplateContent.Common
{
    internal class OutputsHelper
    {
        internal static async Task ProcessOutputs(ServiceBusReceivedMessage message, string deviceId, string messageType, IAsyncCollector<SignalRMessage> signalRMessages, ILogger logger)
        {
            DateTime currentDate = DateTime.UtcNow;

            Parallel.Invoke(
                                    async () =>
                                    {
                                        logger.LogDebug("   >>>PARALLEL PROCESS: SENDING THE MESSAGE TO THE SIGNAL R (OUTPUT)");

                                        string signalRTarget = Environment.GetEnvironmentVariable("afBindingSRTarget", EnvironmentVariableTarget.Process);

                                        if (string.IsNullOrEmpty(signalRTarget))
                                            logger.LogError("   >>>PARALLEL PROCESS: SENDING THE MESSAGE TO THE SIGNAL R (OUTPUT)::No target found.");
                                        else
                                        {
                                            logger.LogDebug("   >>>PARALLEL PROCESS: SENDING THE MESSAGE TO THE SIGNAL R (OUTPUT)::Sending message.");
                                            
                                            //SIGNAL R
                                            //https://docs.microsoft.com/en-us/azure/azure-functions/functions-bindings-signalr-service-output?tabs=in-process&pivots=programming-language-csharp
                                            await signalRMessages.AddAsync(
                                            new SignalRMessage
                                            {
                                                Target = signalRTarget, //client method to be called (also equal to 'category' apparently)
                                                Arguments = new[] {
                                                new {
                                                    deviceId = deviceId,
                                                    messageType = messageType,
                                                    message = JsonConvert.SerializeObject(message.Body),
                                                    date = currentDate
                                                }
                                                }//payload of the target method
                                            });
                                        }

                                    },
                                    async () =>
                                    {
                                        logger.LogDebug("   >>>PARALLEL PROCESS: SENDING THE MESSAGE TO THE STORAGE (OUTPUT)");

                                        //STORAGE                                        
                                        //WARNING: default Blob output binding does not support using the deviceId in the path.
                                        string storageConnectionString = Environment.GetEnvironmentVariable("afBindingOutputStorageConnectionString", EnvironmentVariableTarget.Process);
                                        string storageContainerName = Environment.GetEnvironmentVariable("afBindingValidatedTelemetryStoragePath", EnvironmentVariableTarget.Process);

                                        if (string.IsNullOrEmpty(storageContainerName) && string.IsNullOrEmpty(storageContainerName))
                                        {
                                            logger.LogError("   >>>PARALLEL PROCESS: SENDING THE MESSAGE TO STORAGE (OUTPUT)::No storage configuration has been found.");
                                        }
                                        else
                                        {                                            
                                            BlobServiceClient blobServiceClient = new BlobServiceClient(storageConnectionString);
                                            BlobContainerClient containerClient = blobServiceClient.GetBlobContainerClient(storageContainerName);
                                            BlobClient blobClient = containerClient.GetBlobClient($"{deviceId}/{currentDate.ToString("yyyy-MM-dd")}/{currentDate.ToString("yyyy-MM-ddTHH:mm:ss.fffZ")}-{messageType}.json");

                                            await blobClient.UploadAsync(message.Body, true);

                                            logger.LogDebug("   >>>PARALLEL PROCESS: SENDING THE MESSAGE TO THE STORAGE (OUTPUT)::Message stored.");
                                        }
                                    });
        }
    }
}
