﻿using Azure.Storage.Blobs;

using Microsoft.Extensions.Logging;

using System;
using System.Threading.Tasks;

using TemplateContent.Common.Models;

namespace TemplateContent.Common
{
    internal class SchemaErrorsStorageHelper
    {
        internal static async Task ProcessSchemaErrors(ILogger logger, SchemaValidationResponse response, string deviceId, string correlationId, string messageType)
        {
            if (logger == null)
                throw new ArgumentNullException(nameof(logger));

            if (response == null)
                throw new ArgumentNullException(nameof(response));

            if (string.IsNullOrEmpty(deviceId))
                throw new ArgumentNullException(nameof(deviceId));

            if (string.IsNullOrEmpty(messageType))
                throw new ArgumentNullException(nameof(messageType));


            if (!response.IsMessageValid && response.Errors != null && response.Errors.Count > 0)
            {
                logger.LogDebug("   >>>STORING SCHEMA VALIDATION ERRORS");

                //STORAGE                                        
                //WARNING: default Blob output binding does not support using the deviceId in the path.
                string storageConnectionString = Environment.GetEnvironmentVariable("afBindingOutputStorageConnectionString", EnvironmentVariableTarget.Process);
                string storageContainerName = Environment.GetEnvironmentVariable("afSchemaErrorsStoragePath", EnvironmentVariableTarget.Process);
                DateTime currentDate = DateTime.UtcNow;

                BlobServiceClient blobServiceClient = new BlobServiceClient(storageConnectionString);
                BlobContainerClient containerClient = blobServiceClient.GetBlobContainerClient(storageContainerName);

                string blobName = $"{deviceId}/{currentDate.ToString("yyyy-MM-dd")}/{currentDate.ToString("yyyy-MM-ddTHH:mm:ss.fffZ")}-{messageType}-schemaerrors.json";

                if (!string.IsNullOrEmpty(correlationId))
                    blobName = $"{deviceId}/{currentDate.ToString("yyyy-MM-dd")}/{correlationId}/{currentDate.ToString("yyyy-MM-ddTHH:mm:ss.fffZ")}-{messageType}-schemaerrors.json";

                BlobClient blobClient = containerClient.GetBlobClient(blobName);
                await blobClient.UploadAsync(new BinaryData(response, new System.Text.Json.JsonSerializerOptions { WriteIndented = true }), true);
            }
        }
    }
}
